PROGRAM Compute_Ql_Linear
  USE cosmo
  USE linearpk
  USE growth
  ! A program for computing the cross-power spectrum of CMB lensing potential
  ! and the linear Integrated Sachs-Wolfe (ISW) effect, Q(l). 
  ! The output file (ell_ql.txt) contains:
  ! (1st column) multipole
  ! (2nd column) Q(l)
  ! April 7, 2012: E.Komatsu
  IMPLICIT none
  character(len=128) :: filename
  integer :: lmax=3000 ! maximum multipole
  integer :: lmin=2    ! minimum multipole
  double precision :: zmax=9d0 ! maximum redshift in the input data file
  double precision :: dlnPkdz,kout_ov_h,zin,zout,dln1pz
  double precision :: Ppsi,rombint,rz,rstar,d_chi,ell,dlnell,Ql
  double precision :: g,dgdlna,D,dlnDdz,linear_pk
  integer :: ik,iz,j,n
  external rombint,d_chi,g,dgdlna,linear_pk
! Specify four cosmological parameters
! The data type has been defined in MODULE cosmo.
  ode0=0.723d0
  om0=0.277d0
  h0=0.702d0
  w=-1d0
! tabulate g(z) by calling "setup_growth"
  call setup_growth
! read in and tabulate P(k)
  filename='wmap5baosn_max_likelihood_matterpower_at_z=30.dat'
  n=896 ! # of lines in the file
  zin=30d0
  CALL open_linearpk(filename,n)
! being calculating Q(l)
  filename='ell_ql.txt'
  open(2,file=filename,status='unknown')
  rstar=rombint(d_chi,1d0/1092d0,1d0,1d-6) !comoving distance to z*=1091 [Mpc/h]
  !!$ print*,'r*=',rstar/h0,'Mpc'
  do j=lmin,lmax
     ell=dble(j)
     !!$ print*,'l=',ell
     Ql=0d0
     ! integration over redshift
     do iz=1,100
        if((iz.eq.1).or.(iz.eq.100))then
           dln1pz=0.5d0*dlog((1d0+zmax))/1d2 ! for the integration boundary
        else
           dln1pz=dlog((1d0+zmax))/1d2 ! dlog(1+zmax)/(# of points in redshift)
        endif
        zout=dexp(dble(iz)*dln1pz)-1d0
        rz=rombint(d_chi,1d0/(1d0+zout),1d0,1d-6) ! h^-1 Mpc
        kout_ov_h=ell/rz ! h/Mpc
        D=g(zout)/g(zin)*(1d0+zin)/(1d0+zout) ! linear growth relative to zin
        dlnDdz=-(1d0+dgdlna(zout)/g(zout))/(1d0+zout)
        Ppsi=(1.5d0*om0)**2d0*(1d0/2998d0/kout_ov_h)**4d0 &
             *D**2d0*linear_pk(kout_ov_h)*(1d0+zout)**2d0 ! P(k) of Newtonian potential [h^-3 Mpc^3]
        dlnPkdz=2d0*dlnDdz
        Ql=Ql+2d0*(rstar-rz)/rstar/rz**3d0*Ppsi*(dlnPkdz+2d0/(1d0+zout)) &
             *(1d0+zout)*dln1pz ! cross-power spectrum of lensing-RS effect
     enddo
     !!$ print*,'Ql=',Ql
     write(2,'(1I5,1E18.8)')j,Ql
  enddo
  close(2)
  CALL close_linearpk
END PROGRAM Compute_Ql_Linear
!-----------------------------------------------------------------------------
DOUBLE PRECISION FUNCTION d_chi(x)
! d_chi(x) = 1/[H(x)*x^2] in units of h^-1 Mpc
  USE cosmo
  IMPLICIT none
  DOUBLE PRECISION, intent(IN) :: x
  DOUBLE PRECISION :: tcmb=2.725d0
  DOUBLE PRECISION :: ok0,ogh2,orh2,or0,neff=3.04d0
  ogh2 = 2.469d-5*(tcmb/2.725)**4.
  orh2 = (1.+0.2271073*neff)*ogh2
  or0  = orh2/h0**2.
  ok0  = 1.-om0-ode0 ! don't include or0 in curvature.
  d_chi = 2998d0/sqrt(om0*x+or0+ok0*x**2.+ode0*x**(1.-3.*w))
  return
END FUNCTION d_chi
